@@ -1,3 +1,5 @@
+# python3
+
 import datetime
 import time
 import json
@@ -9,6 +11,8 @@ import requests
 import redis
 from kafka import KafkaProducer
 
+from tisdk.sm4 import encrypt_ecb, decrypt_ecb
+
 
 try:
     requests.packages.urllib3.disable_warnings()
@@ -56,6 +60,7 @@ class TiVendor:
     cache = EasyCache()
     kafka_producers = {}
     redis_pools = {}
+    queue = []
 
     def __init__(self, config):
         self.tiadmin_host = config['TIADMIN_HOST'] or 'http://tiadmin.kong:8000'
@@ -63,7 +68,7 @@ class TiVendor:
         self.kafka_host = config.get('KAFKA_HOST') or 'kafka-kafka.kafka:9092'
         self.kafka_topic = config.get('KAFKA_TOPIC') or 'supplier-request'
         self.default_service_name = config.get('DEFAULT_SERVICE_NAME', '')
-        self.logger_name = config.get('LOGGER_NAME') or None
+        self.logger_name = config.get('LOGGER_NAME') or '__main__'
 
         self.logger = logging.getLogger(self.logger_name)
         self.kafka_producer = self.get_singleton_kafka_producer()
@@ -75,6 +80,8 @@ class TiVendor:
         self.redis_pool = self.get_redis_pool()
         self.redis = None
 
+        self.sm4_secret = config.get('SM4_SECRET', '')
+
     def get_singleton_kafka_producer(self):
 
         if self.kafka_enable != 'on':
@@ -97,6 +104,8 @@ class TiVendor:
         return kafka_producer
 
     def get_api_info(self, api_num):
+        # 缓存 10 分钟
+        
         v_redis = self.get_redis()
         key = f'config:exapi_info:{api_num}'
         info = v_redis.get(key)
@@ -104,14 +113,20 @@ class TiVendor:
             return json.loads(info)
 
         url = self.tiadmin_host + '/supplier/services/'
-        r = requests.get(url, params={'api_num': api_num}).json()
+        
+        r = requests.get(url, params={'api_num': api_num})
+        try:
+            r = r.json()
+        except Exception as e:
+            self.logger.error('parse api info fault,status: %s,resp:%s' % (r.status_code, r.text), exc_info=True)
+            raise e
         results = r.get('results')
         if not results:
             raise TiVendorError(f'{api_num} api not found')
 
         info = results[0]
 
-        v_redis.set(key, json.dumps(info),300)
+        v_redis.set(key, json.dumps(info), 600)
 
         return info
 
@@ -119,8 +134,10 @@ class TiVendor:
         info = self.get_api_info(api_num)
         url = info['url']
         method = info['method']
-        timeout = int(info.get('timeout',60))
-        self.logger.info(f'Vendor Request: {method}, {url}, {kwargs}')
+        timeout = float(info.get('timeout', 60))
+        if timeout <= 0:
+            timeout = 5
+        self.logger.info(f'Vendor Request: {method}, {url}, {timeout}, {kwargs}')
 
         start_time = self.now_time()
         response = requests.request(method, url, timeout=timeout, verify=verify, **kwargs)
@@ -165,8 +182,22 @@ class TiVendor:
                 'service_name': service_name,
             })
 
+            # 判断一下 kong_request_id 如果已经存在就不要重复推送
+            reqid = ti_request_headers.get('Kong-Request-Id', '')
+            for a in TiVendor.queue:
+                # 为加快判断速度，可以先判断最近的几条记录，如果命中就不往下判断了
+                for b in TiVendor.queue[-50:]:
+                    aid = a['ti_request_headers'].get('Kong-Request-Id', '')
+                    bid = b['ti_request_headers'].get('Kong-Request-Id', '')
+                    if aid == reqid:
+                        return
+                    if bid == reqid:
+                        return
+
             start_new_thread(kafka_push, args=(), kwargs=kwargs)
 
+            TiVendor.queue.append(kwargs)
+
     def get_service_name(self):
         try:
             from flask import request
@@ -230,9 +261,26 @@ class TiVendor:
         v_redis.set(cache_key, info.get('source'))
 
         return info.get('source')
+
+
+def desensitization(data, sm4_secret):
+    
+    if '|' not in sm4_secret:
+        return data
+
+    user_md5, key = sm4_secret.split('|')
+    if len(key) != 16:
+        return data
+
+    data = encrypt_ecb(data, key)
+    data = user_md5 + '|' + data
+    return data
+
+
 def kafka_push(**kwargs):
 
     instance = kwargs['instance']
+    
     api_info = kwargs['api_info']
     url = kwargs['url']
     method = kwargs['method']
@@ -243,10 +291,17 @@ def kafka_push(**kwargs):
     service_name = kwargs['service_name']
 
     url_parse = urlparse(url)
+    
     assert isinstance(instance, TiVendor)
-
+    
     request_body = kwargs.get('json', '') or kwargs.get('data', '') or kwargs.get('files', '')
+    request_body = json.dumps(request_body) if isinstance(request_body, (dict, list)) else request_body
+    request_body = desensitization(request_body, instance.sm4_secret)
 
+    request_params = kwargs.get('params', '')
+    request_params = json.dumps(request_params) if isinstance(request_params, (dict, list)) else request_params
+    request_params = desensitization(request_params, instance.sm4_secret)
+    
     body = {
         'start_time': start_time,
         'end_time': end_time,
@@ -260,15 +315,15 @@ def kafka_push(**kwargs):
         'triple_host': '%s:%s' % (url_parse.hostname, url_parse.port) if url_parse.port else url_parse.hostname,
         'method': method,
         'request': {
-            'headers': json.dumps(dict(response.request.headers)),
-            'params': json.dumps(kwargs.get('params', '')),
-            'body': json.dumps(request_body) if isinstance(request_body, dict) else request_body,
+            'headers': '',
+            'params': request_params,
+            'body': request_body,
         },
         'response': {
-            'headers': json.dumps(dict(response.headers)),
-            'body': response.text,
+            'headers': '',
+            'body': '',
         },
-        'status_code': response.status_code,
+        'status_code': 499,
         'service_name': service_name,
         'api_num': api_info.get('api_num', ''),
         'supplier_name': '%s-%s-%s' % (
@@ -276,6 +331,14 @@ def kafka_push(**kwargs):
             api_info.get('product_defines', 'empty')),
     }
 
+    if response is not None:
+        body['request']['headers'] = json.dumps(dict(response.request.headers))
+        body['response'] = {
+            'headers': json.dumps(dict(response.headers)),
+            'body': response.text,
+        }
+        body['status_code'] = response.status_code
+    
     try:
         msg = json.dumps(body).encode('utf8')
         instance.kafka_producer.send(instance.kafka_topic, value=msg)
